/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package filter;

/**
 *
 * @author mm
 */
public class FilterModel {

    static boolean dspBypass;
    /** Flag, true if notchFilter filter is on.  Applies to all DSP filters. */
    static boolean notchFilter;
    /**
     * Noise reduction setting, 1 to 4, least to most aggressive.  Applies
     * to all DSP filters.
     */
    static int noiseReductionLevel;

    /** Flag, true if narrow CW skirts chosem, false if wide. */
    boolean cwNarrow;
    int nrBeta, nrHold, ntBeta, ntHold;
    /** Filter gain. */
    double filterGain;
    /**
     * Center frequency, in Hz.
     */
    double centerFreq;
    /**
     * Denoiser gain.  RTTY does not have nr gain.
     */
    static double denoiserGain;
    /**
     * High frequency cut off, in Hz.
     */
    double highFreqCutoff;
    /**
     * Low frequency cut off, in Hz.
     */
    double lowFreqCutoff;
    /**
     * Filter bandwidth.
     */
    double bandwidth;

    public FilterModel() {
        centerFreq = 600; // Hz;
        cwNarrow = false; // Use wide filter by default.
        dspBypass = false;
        highFreqCutoff = 3500; // Hz
        lowFreqCutoff = 0;
        notchFilter = false;
        noiseReductionLevel = 0;
    }

    public int getNrBeta() {
        return nrBeta;
    }

    public void setNrBeta(int nrBeta) {
        this.nrBeta = nrBeta;
    }

    public int getNrHold() {
        return nrHold;
    }

    public void setNrHold(int nrHold) {
        this.nrHold = nrHold;
    }

    public int getNtBeta() {
        return ntBeta;
    }

    public void setNtBeta(int ntBeta) {
        this.ntBeta = ntBeta;
    }

    public int getNtHold() {
        return ntHold;
    }

    public void setNtHold(int ntHold) {
        this.ntHold = ntHold;
    }

    public boolean isDspBypass() {
        return dspBypass;
    }

    public void setDspBypass(boolean dspBypass) {
        FilterModel.dspBypass = dspBypass;
    }

    public double getDenoiserGain() {
        return denoiserGain;
    }

    public void setDenoiserGain(double denoiserGain) {
        this.denoiserGain = denoiserGain;
    }

    public double getFilterGain() {
        return filterGain;
    }

    public void setFilterGain(double filterGain) {
        this.filterGain = filterGain;
    }

    public double getBandwidth() {
        return bandwidth;
    }

    public void setBandwidth(double bandwidth) {
        lowFreqCutoff = centerFreq - bandwidth / 2;
        highFreqCutoff = centerFreq + bandwidth / 2;
        this.bandwidth = bandwidth;
    }

    public double getCenterFreq() {
        return centerFreq;
    }

    public void setCenterFreq(double centerFreq) {
        lowFreqCutoff = centerFreq - bandwidth / 2;
        highFreqCutoff = centerFreq + bandwidth / 2;
        this.centerFreq = centerFreq;
    }

    public boolean isCwNarrow() {
        return cwNarrow;
    }

    public void setCwNarrow(boolean cwNarrow) {
        this.cwNarrow = cwNarrow;
    }

    public double getHighFreqCutoff() {
        return highFreqCutoff;
    }

    public void setHighFreqCutoff(double highFreqCutoff) {
        bandwidth = highFreqCutoff - lowFreqCutoff;
        centerFreq = lowFreqCutoff + bandwidth / 2;
        this.highFreqCutoff = highFreqCutoff;
    }

    public double getLowFreqCutoff() {
        return lowFreqCutoff;
    }

    public void setLowFreqCutoff(double lowFreqCutoff) {
        bandwidth = highFreqCutoff - lowFreqCutoff;
        centerFreq = lowFreqCutoff + bandwidth / 2;
        this.lowFreqCutoff = lowFreqCutoff;
    }

    public int getNoiseReductionLevel() {
        return noiseReductionLevel;
    }

    public void setNoiseReductionLevel(int noiseReductionLevel) {
        this.noiseReductionLevel = noiseReductionLevel;
    }

    public boolean isNotchFilter() {
        return notchFilter;
    }

    public void setNotchFilter(boolean notchFilter) {
        this.notchFilter = notchFilter;
    }

    public String toString() {
        String s = "Filter:\n"
                + "  Center freq:  " + centerFreq + "\n"
                + "  Bandwidth:    " + bandwidth + "\n"
                + "  Lo freq cut:  " + lowFreqCutoff + "\n"
                + "  Hi freq cut:  " + highFreqCutoff + "\n"
                + "  NR:           " + noiseReductionLevel + "\n"
                + "  NR beta/hold: " + nrBeta + "/" + nrHold + "\n"
                + "  Notch:        " + notchFilter + "\n"
                + "  Nt beta/hold: " + ntBeta + "/" + ntHold + "\n"
                + "  DSP bypass:   " + dspBypass;
        return s;
    }
}
