package elecraft.util;
import java.io.*;
import java.util.*;

/**
 *	For some insane reason, the Java language designers believe unsigned
 *	numbers are too difficult for programmers to understand.  So
 *	we're stuck implementing our own...  :-/  That means for an array of
 *	bytes, if the MSB is set you get a 2's complement negative number.  The
 *	only way around it I've discovered is to grab the array of bytes and
 *	stuff them into an array of ints while masking all but the least
 *	significant 8 bits.  Grumble, grumble.
 */

public class Unsigned {

	Unsigned() {}

	public static int[] bytes2UInts(byte[] bytes) {
		
		int n = bytes.length;
		int[] ints = new int[n];
		for (int i = 0; i < n; i++)
			ints[i] = 0xff & (int) bytes[i];

		return ints;
	}

	public static long byte2UInt(byte[] buf, int start) {

		//System.out.println("buf[1]: " +Integer.toHexString(buf[start]));
		//System.out.println("buf[2]: " +Integer.toHexString(buf[start+1]));
		//System.out.println("buf[3]: " +Integer.toHexString(buf[start+2]));
		//System.out.println("buf[4]: " +Integer.toHexString(buf[start+3]));

		int firstByte  = (0xff & ((int) buf[start]));
		int secondByte = (0xff & ((int) buf[start+1]));
		int thirdByte  = (0xff & ((int) buf[start+2]));
		int fourthByte = (0xff & ((int) buf[start+3]));
		long number  = ((long) (firstByte  << 24 | secondByte << 16
			| thirdByte  << 8 | fourthByte)) & 0xffffffffL;

		return number;

	}

	public static void uInt2Byte(long number, byte[] buf, int start) {

		buf[start]   = (byte) ((number & 0xff000000L) >> 24);
		buf[start+1] = (byte) ((number & 0x00ff0000L) >> 16);
		buf[start+2] = (byte) ((number & 0x0000ff00L) >> 8);
		buf[start+3] = (byte) ((number & 0x000000ffL));
	}

	public static int byte2UShort(byte[] buf, int start) {

		int firstByte =  (0x000000ff & ((int) buf[start]));
        int secondByte = (0x000000ff & ((int) buf[start+1]));
		int number = (int) (firstByte << 8 | secondByte);
		return number;
	}

	public static void uShort2Byte(int number, byte[] buf, int start) {
		buf[start] =   (byte) ((number & 0xff00) >> 8);
		buf[start+1] = (byte) (number & 0x00ff);
	}

	public static int byte2UByte(byte[] buf, int start) {

		int firstByte = (0x000000ff & ((int)buf[start]));
		int number = (short) firstByte;

		return number;
	}

	public static void uByte2Byte(int number, byte[] buf, int start) {
		buf[start] = (byte) (number & 0xff);
	}

}
