package k2;

/** Create a K2 RIT/XIT offset knob.  The knob is modeled to have 20 discrete
 * steps.  That is, instead if splitting the circle into 360 degrees or 2pi
 * radians, here it is split into 20 steps.  Each step moves the RIT up or
 * down by the expected amount depending on whether Fine RIT or normal RIT is
 * engaged.
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br>
 * March 2007
 */

public class KnobOffset extends Knob {

	private double stepsMoved;

	/** Create knob with specified center and radius, in pixels. */
	KnobOffset(int cx, int cy, int r) {
		ctrX = cx;
		ctrY = cy;
		radius = r;
		stepsMoved = 0.;
	}

	/** Get the K2 command to turn the knob up or down one step. */
	public String getClearCommand() {
		return "RC;";
	}

	/** Get the K2 command to turn the knob up or down one step. */
	public String getK2Command() {
		String cmd;

		// Tune up/down as appropriate.
		if (getTurningDir() == -1)
			cmd = "RD;";
		else
			cmd = "RU;";

		return cmd;
	}
}
