package util;

import gnu.io.CommPortIdentifier;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.LinkedList;

public class Serial {

    CommPortIdentifier portId;
    InputStream inputStream;
    OutputStream outputStream;
    SerialPort serialPort;
    static Enumeration portList;

    public Serial() {
        inputStream = null;
        outputStream = null;
    }

    public boolean findK3() {

        portId = null;
        PortList llcp = new PortList();

        portList = CommPortIdentifier.getPortIdentifiers();
        while (portList.hasMoreElements()) {
            llcp.add((CommPortIdentifier) portList.nextElement());
        }
        llcp.sort();
        for (CommPortIdentifier cpid : llcp) {

            portId = cpid;
            serialPort = null;
            try {
                serialPort = (SerialPort) portId.open("K3", 2000);
            } catch (PortInUseException e) {
                System.out.println("Port " + portId.getName() + " in use.");
            }

            if (serialPort == null) {
                continue;
            }
            try {
                serialPort.setSerialPortParams(
                        // Set up per Elecraft K3 Owner's manual.
                        38400,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
            } catch (UnsupportedCommOperationException e) {
                System.out.println("Couldn't open " + portId.getName() + ".");
            }

            try {
                inputStream = serialPort.getInputStream();
                outputStream = serialPort.getOutputStream();
            } catch (IOException e) {
            }

            // Try sending a command to the K3.
            // There's always a response, so let's get it now.
            byte[] response = new byte[8];
            int timeout = 100;
            serialWrite("K31;K3;", timeout);
            serialRead(response);
            String isK3 = new String(response, 0, 4);
            if (isK3.equals("K31;")) {
                System.out.println("Found K3 on port " + portId.getName() + ".");
                return true;
            } else {
                System.out.println(portId.getName() + ": response: '"
                        + response + "'");
            }
            close();
        }

        System.out.println("Didn't find K3!");
        return false;
    }

    public boolean findW1() {

        portId = null;
        PortList llcp = new PortList();

        portList = CommPortIdentifier.getPortIdentifiers();
        while (portList.hasMoreElements()) {
            llcp.add((CommPortIdentifier) portList.nextElement());
        }
        llcp.sort();
        for (CommPortIdentifier cpid : llcp) {

            portId = cpid;
            serialPort = null;
            try {
                serialPort = (SerialPort) portId.open("W1", 2000);
            } catch (PortInUseException e) {
                System.out.println("Port " + portId.getName() + " in use.");
            }

            if (serialPort == null) {
                continue;
            }
            try {
                serialPort.setSerialPortParams(
                        // Set up per Elecraft W1 manual.
                        9600,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);

                serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
            } catch (UnsupportedCommOperationException e) {
                System.out.println("Couldn't open " + portId.getName() + "!");
            }

            try {
                inputStream = serialPort.getInputStream();
                outputStream = serialPort.getOutputStream();
            } catch (IOException e) {
            }

            // Try sending a command to the W1.
            int timeout = 60;
            serialWrite("V", timeout);
            // There's always a response, so let's get it now.
            byte[] response = new byte[8];
            serialRead(response);
            String version = new String(response, 0, 5);
            if (version.equals("V1.05")) {
                System.out.println("Found W1 on port " + portId.getName() + ".");
                return true;
            }
            close();
        }

        return false;
    }

    public void close() {
        if (inputStream != null) {
            try {
                inputStream.close();
            } catch (IOException ex) {
            }
        }
        if (outputStream != null) {
            try {
                outputStream.close();
            } catch (IOException ex) {
            }
        }
        if (serialPort != null) {
            serialPort.close();
        }
    }

    private class PortList extends LinkedList<CommPortIdentifier> {

        public void sort() {
            Collections.sort(this, new Comparator<CommPortIdentifier>() {

                @Override
                public int compare(CommPortIdentifier o1, CommPortIdentifier o2) {

                    return o1.getName().compareTo(o2.getName());

//                    int numBeg1 = firstNum(o1.getName());
//                    int numBeg2 = firstNum(o2.getName());
//                    if (numBeg1 == -1 || numBeg2 == -1) {
//                        if (o1.getName().compareTo(o2.getName()) > 0) {
//                            return 1;
//                        } else if (o1.getName().compareTo(o2.getName()) == 0) {
//                            return 0;
//                        } else {
//                            return -1;
//                        }
//                    }
//                    int numEnd1 = lastNum(o1.getName());
//                    int numEnd2 = lastNum(o2.getName());
//                    int num1 = Integer.valueOf(
//                            o1.getName().substring(numBeg1, 1 + numEnd1));
//                    int num2 = Integer.valueOf(
//                            o2.getName().substring(numBeg2, 1 + numEnd2));
//                    if (num1 < num2) {
//                        return -1;
//                    } else if (num1 == num2) {
//                        return 0;
//                    } else {
//                        return 1;
//                    }
                }
            });
        }

        private int firstNum(String s) {
            for (int i = 0; i < s.length(); i++) {
                switch (s.charAt(i)) {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        return i;
                    default:
                        break;
                }
            }
            return -1;
        }

        private int lastNum(String s) {
            for (int i = s.length() - 1; i >= 0; i--) {
                switch (s.charAt(i)) {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        return i;
                    default:
                        break;
                }
            }
            return -1;
        }
    }

    public boolean setSerialPort(String app, String portName) {
        close();
        portId = findSerialPort(portName);
        return serialSetup(app, portId);
    }

    public CommPortIdentifier findSerialPort(String portName) {

        portId = null;

//        if (portList == null) {
        portList = CommPortIdentifier.getPortIdentifiers();
//        }
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL
                    && portId.getName().equals(portName)) {
                return portId;
            }
        }
        System.out.println("Port " + portName + " not found.");
        return null;
    }

    public String[] getSerialPortNames() {

        PortList llcp = new PortList();
        portList = CommPortIdentifier.getPortIdentifiers();
        while (portList.hasMoreElements()) {
            llcp.add((CommPortIdentifier) portList.nextElement());
        }
        llcp.sort();
        int len = llcp.size();
        String[] names = new String[len];
        for (int i = 0; i < len; i++) {
            names[i] = llcp.get(i).getName();
        }
        return names;
    }

    private boolean serialSetup(String app, CommPortIdentifier portId) {

        SerialPort serialPort = null;

        try {
            serialPort = (SerialPort) portId.open(app, 2000);
        } catch (PortInUseException e) {
            System.out.println(getClass() + ".serialSetup: couldn't open "
                    + portId.getName() + ".");
            return false;
        }

        try {
            inputStream = serialPort.getInputStream();
            outputStream = serialPort.getOutputStream();
        } catch (IOException e) {
            System.out.println(getClass() + ".serialSetup: couldn't get "
                    + "input & output streams.");
            return false;
        }

        try {
            if (app.equals("K2")) {
                serialPort.setSerialPortParams(
                        // Set up per Elecraft KIO2 manual.
                        4800,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_2,
                        SerialPort.PARITY_NONE);
                serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
            } else if (app.equals("K3") || app.equals("K3info")) {
                serialPort.setSerialPortParams(
                        // Set up per Elecraft K3 Owner's manual.
                        38400,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
                serialPort.setRTS(false);
                serialPort.setDTR(false);
            } else if (app.equals("W1") || app.equals("W2")) {
                serialPort.setSerialPortParams(
                        // Set up per Elecraft W1 and W2 manuals.
                        9600,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
            } else {
                System.out.println("Unrecognized app.");
                return false;
            }
            return true;
        } catch (UnsupportedCommOperationException e) {
            System.out.println("Couldn't open serial port!");
            return false;
        }
    }

    public byte[] serialRead(byte[] response) {

        if (inputStream == null) {
            return null;
        }

        int estimate = 0;
        try {
            estimate = inputStream.available();
        } catch (IOException e) {
            System.out.println(getClass() + ".serialRead: 0 bytes waiting.");
        }

        if (estimate > 0) {
            try {
                inputStream.read(response);
            } catch (IOException e) {
                System.out.println(getClass() + ".serialRead: bit of an issue reading!");
            }
        } else {
            for (int i = 0; i < response.length; i++) {
                response[i] = (byte) ' ';
            }
        }
        String rsp = new String(response);
        int firstSemiColon = rsp.indexOf(';');
        int secondSemiColon = rsp.lastIndexOf(';');
        if (firstSemiColon != secondSemiColon) {
            rsp = rsp.substring(firstSemiColon + 1, secondSemiColon);
            byte bytes[] = rsp.getBytes();
            for (int i = 0; i < bytes.length; i++) {
                response[i] = (byte) rsp.charAt(i);

            }
        }
        return response;
    }

    public void serialWrite(byte[] cmd) {

        if (outputStream == null) {
            return;
        }

        try {
            outputStream.write(cmd);
        } catch (IOException e) {
        }
    }

    public void serialWrite(byte[] cmd, int delay) {

        if (outputStream == null) {
            return;
        }

        try {
            outputStream.write(cmd);
        } catch (IOException e) {
            System.out.println(getClass() + ".serialWrite: write exception.");
        }

        try {
            // Elecraft recommends a 100 msec wait after a write,
            // though they also say most commands will be processed
            // within 20 msec.
            Thread.sleep(delay);
        } catch (InterruptedException e) {
            System.out.println(getClass() + ".serialWrite: sleep interrupted.");
        }
    }

    public void serialWrite(String s) {
        serialWrite(s.getBytes());
    }

    public void serialWrite(String s, int delay) {
        serialWrite(s.getBytes(), delay);
    }

    public String cmdResp(String cmd, boolean expectReply,
            int msecDelay, int numTries) {
        for (int i = 0; i < numTries; i++) {
            serialWrite(cmd, msecDelay);
            byte[] resp = new byte[64];
            serialRead(resp);
            String response = new String(resp);
            if (!expectReply) {
                return "NA";
            }
            if (!cmd.substring(0, 2).equals(response.substring(0, 2))) {
                continue;
            }
            return response;
        }
        return null;
    }

    public byte[] cmdResp(String cmd, int msecDelay, int numTries) {
        for (int i = 0; i < numTries; i++) {
            serialWrite(cmd, msecDelay);
            byte[] resp = new byte[64];
            serialRead(resp);
            String response = new String(resp);
            if (!cmd.substring(0, 2).equals(response.substring(0, 2))) {
                continue;
            }
            return resp;
        }
        return null;
    }
}
