package digital;
import java.awt.Rectangle;

/** The Button class is a simple model of the geometry and function of a
 * button on the Elecraft W1 power meter.  A button has two functions, one
 * when the user clicks it and another when the user clicks and holds.  The
 * Button object refers to them as the primary and secondary functions.
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br>
 * March 2007 */

public class Button {

	char primary;
	char secondary;
	Rectangle geometry;
	String name;
	String pstr;
	String sstr;

	Button() {
	}

	/** The Button constructor uses W1 functions and button geometry.  The
	 * remaining arguments describe where on the image, in pixels, is this
	 * particular button. */

	Button(String name, int x0, int y0, int xlen, int ylen) {
		this.name = name;
		geometry = new Rectangle(x0, y0, xlen, ylen);
	}

	/** Return the name of this button.  This name is not displayed on-screen
	 * but used internally as an identifier. */
	public String getName() { return name; }
	/** Set the internal name of this button. */
	public void setName(String s) { name = new String(s); }

	/** Indicate of this button's geometry contains the point (pixels). */
	public boolean contains(int x, int y) { return geometry.contains(x, y); }

	/** Return the button's location and size, in pixels. */
	public Rectangle getGeometry() { return geometry; }
	/** Specify the button's location and size, in pixels. */
	public void setGeometry(Rectangle r) { geometry = r; }
	/** Specify the button's location and size, in pixels. */
	public void setGeometry(int x0, int y0, int xlen, int ylen) {
		geometry = new Rectangle(x0, y0, xlen, ylen);
	}

	/** Specify button's primary function.  It's convenient to use the W1's
	 * single letter commands representing the function.  Then the retrieved
	 * button name can be sent over the serial link to the W1. */
	public void setPrimary(char setting) { primary = setting; }
	/** Retrieve button's primary function. */
	public char getPrimary() { return primary; }
	/** Specify button's secondary function.  */
	public void setSecondary(char setting) { secondary = setting; }
	/** Retrieve button's secondary function. */
	public char getSecondary() { return secondary; }

	/** Get the name of the button's primary function.  This label will be
	 * displayed on the virtual W1 screen indicating W1 state. */
	public String getPrimaryLabel() { return pstr; }
	/** Get the name of the button's secondary function. */
	public String getSecondaryLabel() { return sstr; }

	/** Return's the W1 command that executes this button's primary function. */
	public String getTapFn() { return ""; }
	/** Return's the W1 command that executes this button's secondary
	* function. */
	public String getHoldFn() { return ""; }
}
