package digital;
import java.awt.Rectangle;

/** Implement the range/drop button on the front of the virtual W1.  The
 * implementation cycles through the auto, low, medium, high scale ranges and
 * the secondary function of cycling through speed of scale range changes.
 * The latter is so that the meter doesn't change scales briefly just because
 * a transient reading went briefly off-scale.
 *
 * Mike Markowski<br>
 * March 2007
 */

public class ButtonRange extends Button {

	/** Create a range/drop button specifying internally used button name (not
	 * displayed on-screen) and the geometry in pixels specifying where the
	 * button is on-screen. */
	ButtonRange(String name, int x0, int y0, int xlen, int ylen) {
		this.name = name;
		geometry = new Rectangle(x0, y0, xlen, ylen);
		primary = '0'; // Autorange.
		secondary = 'M'; // Medium range drop rate.
		pstr = "AUTO";
		sstr = "MED";
	}

	/** For each tap, i.e., call to this method, the ButtonRange cycles
	 * through low, medium, high scales for measurement display.  The
	 * appropriate W1 single letter command is returned to put the W1 in
	 * the requested scale. */
    @Override
	public String getTapFn() {
		switch (primary) {
		case '0':
			primary = '1';
			pstr = "LO";
			return "1";
		case '1':
			primary = '2';
			pstr = "MED";
			return "2";
		case '2':
			primary = '3';
			pstr = "HI";
			return "3";
		case '3':
			primary = '0';
			pstr = "AUTO";
			return "0";
		}
		System.out.println("RANGE shouldn't be here!");
		return null;
	}

	/** For each hold, i.e., call to this method, the ButtonRange cycles
	 * through slow, medium, fast scales for scale change rate.  The
	 * appropriate W1 single letter command is returned to put the W1 at
	 * the requested range change rate. */
    @Override
	public String getHoldFn() {
		switch (secondary) {
		case 'S':
			secondary = 'M';
			sstr = "MED";
			return "8";
		case 'M':
			secondary = 'F';
			sstr = "FAST";
			return "9";
		case 'F':
			secondary = 'S';
			sstr = "SLO";
			return "7";
		default:
			System.out.println("DROP shouldn't be here!");
			return null;
		}
	}

}
